package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import djmil.cordacheckers.checkers.Stone;
import djmil.cordacheckers.contracts.GameProposalContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;

@BelongsToContract(GameProposalContract.class)
public class GameProposalState extends GameState {

    private final MemberX500Name issuerName;
    private final Map<Integer, Stone> board;

    @ConstructorForDeserialization
    public GameProposalState(MemberX500Name whitePlayer, MemberX500Name blackPlayer, MemberX500Name issuerName, 
            UUID gameUuid, Map<Integer, Stone> board, String message, List<PublicKey> participants) {
        super(whitePlayer, blackPlayer, gameUuid, message, participants);

        this.issuerName = issuerName;
        this.board = board;
    }

    public MemberX500Name getIssuerName() {
        return issuerName;
    }

    public MemberX500Name getAcquierName() {
        return getOpponentName(issuerName);
    }

    public Map<Integer, Stone> getBoard() {
        return Collections.unmodifiableMap(board);
    }

}
